import type {
  ExpandedMergeRequestSchema,
  MergeRequestChangesSchema,
  MergeRequestDiffVersionsSchema,
  MergeRequestNoteSchema,
} from "@gitbeaker/rest"

import GitlabBaseService from "./base"

class MergeRequests extends GitlabBaseService {
  /**
   * 获取合并请求的评论，支持分页。
   * @param {number} [page=1] - 页码。
   * @param {number} [per_page=20] - 每页的评论数。
   * @returns {Promise<MergeRequestNoteSchema[]>} 返回包含评论的Promise。
   */
  async getComments(
    page: number = 1,
    per_page: number = 20
  ): Promise<MergeRequestNoteSchema[]> {
    const URL = `/projects/${this.project_id}/merge_requests/${this.merge_request_iid}/notes?page=${page}&per_page=${per_page}`
    return this.errorWarp<MergeRequestNoteSchema[]>(() => this.get(URL), [])
  }

  /**
   * 获取合并请求的变更。
   * @returns {Promise<MergeRequestChangesSchema | null>} 返回包含变更的Promise。
   */
  async getChanges(): Promise<MergeRequestChangesSchema | null> {
    const URL = `/projects/${this.project_id}/merge_requests/${this.merge_request_iid}/changes`
    return this.errorWarp<MergeRequestChangesSchema | null>(
      () => this.get(URL),
      null
    )
  }

  /**
   * 获取合并请求的详细信息。
   * @returns {Promise<ExpandedMergeRequestSchema | null>} 返回包含详细信息的Promise。
   */
  async getDetail(): Promise<ExpandedMergeRequestSchema | null> {
    const URL = `/projects/${this.project_id}/merge_requests/${this.merge_request_iid}`
    return this.errorWarp<ExpandedMergeRequestSchema | null>(
      () => this.get(URL),
      null
    )
  }

  /**
   * 获取合并请求的差异版本。
   * @returns {Promise<MergeRequestDiffVersionsSchema[]>} 返回包含差异版本的Promise。
   */
  async getDiffVersions(): Promise<MergeRequestDiffVersionsSchema[]> {
    const URL = `/projects/${this.project_id}/merge_requests/${this.merge_request_iid}/versions`
    return this.errorWarp<MergeRequestDiffVersionsSchema[]>(
      () => this.get(URL),
      []
    )
  }
}

export default MergeRequests
